function MapWidget( baseWMS, aoiMinLon, aoiMinLat, aoiMaxLon, aoiMaxLat, viewMinLon, viewMinLat, viewMaxLon, viewMaxLat )
{
  this.aoiLayer = null;
  this.polygonControl = null;
  this.map = null;
  this.dataLayer = null;

  this.baseWMS = baseWMS;
  this.dataWMS = null;


  this.aoiMinLon = aoiMinLon;
  this.aoiMinLat = aoiMinLat;
  this.aoiMaxLon = aoiMaxLon;
  this.aoiMaxLat = aoiMaxLat;

  this.viewMinLon = viewMinLon;
  this.viewMinLat = viewMinLat;
  this.viewMaxLon = viewMaxLon;
  this.viewMaxLat = viewMaxLat;
}

MapWidget.prototype.changeMapSize = function ()
{
  /*
   var clientWidth = Dom.getViewportWidth();
   var clientHeight = Dom.getViewportHeight();

   var mapWidth = $("mapTitle").offsetWidth;
   var mapHeight = clientHeight - $("hd").offsetHeight - $("ft").offsetHeight;

   $("map").style.width = mapWidth + "px";
   $("map").style.height = mapHeight + "px";

   map.updateSize();

   */
  var mapTitle = $("mapTitle");
  var mapDiv = $("map");

  mapDiv.style.width = mapTitle.offsetWidth + "px";
  mapDiv.style.height = Math.round(mapTitle.offsetWidth / 2) + "px";

  //alert(mapDiv.style.width + " " + mapDiv.style.height);

  if ( this.map )
  {
    this.map.updateSize();
  }
};

MapWidget.prototype.setCenterText = function( e )
{
  if ( this.map )
  {
    var center = this.map.getCenter();
    $("centerLon").value = center.lon;
    $("centerLat").value = center.lat;
  }
};


MapWidget.prototype.setupMapWidget = function()
{
  var eventListeners = {
    "moveend": this.setCenterText,
    "zoomend": this.setView
    //"changelayer": mapLayerChanged,
    //"changebaselayer": mapBaseLayerChanged
  };

  this.map = new OpenLayers.Map("map", { controls: [] /*, eventListeners: eventListeners*/ });
  this.map.addControl(new OpenLayers.Control.LayerSwitcher());
  this.map.addControl(new OpenLayers.Control.MousePosition());
  this.map.addControl(new OpenLayers.Control.Scale());
  this.map.addControl(new OpenLayers.Control.ScaleLine());
};


MapWidget.prototype.setView = function()
{
  if ( this.map )
  {
    var bounds = this.map.getExtent();

    $("viewMinLon").value = bounds.left;
    $("viewMaxLat").value = bounds.top;
    $("viewMaxLon").value = bounds.right;
    $("viewMinLat").value = bounds.bottom;
  }
  else
  {
    alert(this.map);
  }
};

MapWidget.prototype.setupBaseLayer = function ()
{
  var baseLayer = new OpenLayers.Layer.WMS(
      this.baseWMS.title,
      this.baseWMS.url,
  {layers: this.baseWMS.layers, format: 'image/jpeg' },
  {buffer:1}
      );
  this.map.addLayer(baseLayer);
  this.map.setBaseLayer(baseLayer);
};


MapWidget.prototype.setupAoiLayer = function ()
{

  this.aoiLayer = new OpenLayers.Layer.Vector("Area of Interest");
  this.aoiLayer.events.register("featureadded", this.aoiLayer, this.setAOI);

  var polyOptions = {sides: 4, irregular: true} ;

  this.polygonControl = new OpenLayers.Control.DrawFeature(this.aoiLayer, OpenLayers.Handler.RegularPolygon,
  {handlerOptions: polyOptions});

  this.map.addLayer(this.aoiLayer);
  this.map.addControl(this.polygonControl);

  if ( this.aoiMinLon && this.aoiMinLat && this.aoiMaxLon && this.aoiMaxLat )
  {
    var bounds = new OpenLayers.Bounds(this.aoiMinLon, this.aoiMinLat, this.aoiMaxLon, this.aoiMaxLat);
    var feature = new OpenLayers.Feature.Vector(bounds.toGeometry());

    this.aoiLayer.addFeatures(feature, {silent: true});
  }
};

MapWidget.prototype.setupToolbar = function ()
{
  var zoomBox = new OpenLayers.Control.ZoomBox(
  {title:"Zoom into an area by clicking and dragging"});


  var polyOptions = {sides: 4, irregular: true};

  var polygonControl = new OpenLayers.Control.DrawFeature(this.aoiLayer, OpenLayers.Handler.RegularPolygon,
  {handlerOptions: polyOptions, title: "Specify Area of Interest"});


  var button1 = new OpenLayers.Control.Button({title:'Clear Area of Interest',
    displayClass: "olControlClearAreaOfInterest",
    trigger: this.clearAOI

  });


  var container = $("panel2");

  var panel = new OpenLayers.Control.Panel(
  { div: container,defaultControl: zoomBox,'displayClass': 'olControlPanel'}
      );


  var nav = new OpenLayers.Control.NavigationHistory({
    nextOptions: {title: "Next View"},
    previousOptions: {title: "Previous View"}
  });


  this.map.addControl(nav);

  panel.addControls([
    new OpenLayers.Control.MouseDefaults({title:'Drag to recenter map'}),
    zoomBox,
    nav.next, nav.previous,
    new OpenLayers.Control.ZoomToMaxExtent({title:"Zoom to the max extent"}),
    this.polygonControl,
    button1
  ]);

  this.map.addControl(panel);
};

MapWidget.prototype.clearAOI = function ()
{
  // HACK - Need a better way to this  
  this.aoiLayer.destroyFeatures();


  // HACK - Need a better way to this
  $("aoiMinLon").value = "";
  $("aoiMaxLat").value = "";
  $("aoiMaxLon").value = "";
  $("aoiMinLat").value = "";
};

MapWidget.prototype.setAOI = function ( e )
{
  var geom = e.feature.geometry;
  var bounds = geom.getBounds();
  var feature = new OpenLayers.Feature.Vector(geom);

  // HACK - Need a better way to this
  $("aoiMinLon").value = bounds.left;
  $("aoiMaxLat").value = bounds.top;
  $("aoiMaxLon").value = bounds.right;
  $("aoiMinLat").value = bounds.bottom;

  this.aoiLayer.destroyFeatures();
  this.aoiLayer.addFeatures(feature, {silent: true});
};

MapWidget.prototype.setupMapView = function ()
{
  var bounds = new OpenLayers.Bounds(this.viewMinLon, this.viewMinLat, this.viewMaxLon, this.viewMaxLat);
  var zoom = this.map.getZoomForExtent(bounds, true);

  //alert(zoom + " " + this.viewMinLon + " " + this.viewMinLat + " " + this.viewMaxLon + " " + this.viewMaxLat);

  this.map.setCenter(bounds.getCenterLonLat(), zoom);

};


MapWidget.prototype.setupCallbacks = function ()
{
  this.map.events.register("moveend", this.map, this.setCenterText);
  this.map.events.register("zoomend", this.map, this.setView);
};

var Dom = YAHOO.util.Dom;
var Event = YAHOO.util.Event;

Event.onDOMReady(function()
{
  init();
});
